'' =================================================================================================
''
''   File....... jm_rcx_check.spin
''   Purpose.... Look for activity from RC receiver
''   Author..... Jon McPhalen
''               Copyright (c) 2025 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 04 JAN 2025
''
'' =================================================================================================


con { timing }

  _xinfreq = 5_000_000                                          ' use 5MHz crystal
  _clkmode = xtal1 + pll16x                                     ' x16 = 80MHz

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate


con { fixed io pins }

  PGM_RX   = 31  { I }                                          ' serial / programming
  PGM_TX   = 30  { O }

  EE_SDA   = 29  { I/O }                                        ' i2c / eeprom
  EE_SCL   = 28  { I/O }


con { app io pins }

  RCX      = 16  { I }                                          ' input from RC receiver


con

  #true,  ON, OFF
  #false, NO, YES

  USE_SIM = YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  prng : "jm_prng"                                              '   pseudo-random number generation
  io   : "jm_io"                                                '   simple io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal

' * uses cog when loaded


dat { pre-initialized }


var { globals }


pub main | us

  setup
  wait_for_terminal(true, 250)

  term.str(string("Checking for RC input", 13))

  if (rcx_active(RCX))
    term.str(string("-- Present", 13))
    us := io.pulse_in(RCX, 1)
    term.str(string("-- "))
    term.dec(us / US_001)
  else
    term.str(string("-- Not detected", 13))

  repeat
    waitcnt(0)


pub rcx_active(pin) : result | t

'' Looks for servo pulse on pin

  dira[pin] := 0                                                ' force to input
  ctra := (%01000 << 26) | pin                                  ' set POS detect
  frqa := 1
  phsa := 0

  waitcnt(cnt + (clkfreq / 50))                                 ' wait 20ms

  return (phsa > 0)                                             ' true if activity detected


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer

  io.start(0, 0)                                                ' clear all pins (master cog)

  term.tstart(BR_TERM)                                          ' start serial for terminal *

  if (USE_SIM == YES)                                           ' start simulator cog?
    cognew(pulse_gen(RCX), @stack)


pub wait_for_terminal(clear, delay)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)
    time.pause(delay #> 0)


con { background pulse generator }

' *** DO NOT START WITH RC RECEIVER ATTACHED ***


var

  long  stack[64]


pri pulse_gen(pin) | t

  outa[pin] := 0                                                ' make output low (this cog)
  dira[pin] := 1

  t := cnt                                                      ' sync 20ms loop
  repeat
    outa[pin] := 1                                              ' high
    waitcnt(t + constant(1500 * US_001))                        ' hold 1500us
    outa[pin] := 0                                              ' low
    waitcnt(t += constant(20 * MS_001))                         ' let 20ms loop finish


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}